from typing import IO

from PIL.Image import Image as _PILImage
from willow.image import (
    AvifImageFile,
    GIFImageFile,
    HeicImageFile,
    IcoImageFile,
    Image,
    ImageFile,
    JPEGImageFile,
    PNGImageFile,
    RGBAImageBuffer,
    RGBImageBuffer,
    WebPImageFile,
)

class UnsupportedRotation(Exception): ...

class PillowImage(Image):
    image: _PILImage
    def __init__(self, image: _PILImage) -> None: ...
    @classmethod
    def check(cls) -> None: ...
    @classmethod
    def is_format_supported(cls, image_format: str) -> bool: ...
    def get_size(self) -> tuple[int, int]: ...
    def get_frame_count(self) -> int: ...
    def has_alpha(self) -> bool: ...
    def has_animation(self) -> bool: ...
    def resize(self, size: tuple[int, int]) -> PillowImage: ...
    def crop(self, rect: tuple[int, int, int, int]) -> PillowImage: ...
    def rotate(self, angle: int) -> PillowImage: ...
    def set_background_color_rgb(self, color: tuple[int, int, int] | list[int]) -> PillowImage: ...
    def get_icc_profile(self) -> bytes | None: ...
    def get_exif_data(self) -> bytes | None: ...
    def transform_colorspace_to_srgb(self, rendering_intent: int = 0) -> PillowImage: ...
    def save_as_jpeg(
        self,
        f: IO[bytes],
        quality: int = 85,
        optimize: bool = False,
        progressive: bool = False,
        apply_optimizers: bool = True,
    ) -> JPEGImageFile: ...
    def save_as_png(
        self,
        f: IO[bytes],
        optimize: bool = False,
        apply_optimizers: bool = True,
    ) -> PNGImageFile: ...
    def save_as_gif(self, f: IO[bytes], apply_optimizers: bool = True) -> GIFImageFile: ...
    def save_as_webp(
        self,
        f: IO[bytes],
        quality: int = 80,
        lossless: bool = False,
        apply_optimizers: bool = True,
    ) -> WebPImageFile: ...
    def save_as_heic(
        self,
        f: IO[bytes],
        quality: int = 80,
        lossless: bool = False,
        apply_optimizers: bool = True,
    ) -> HeicImageFile: ...
    def save_as_avif(
        self,
        f: IO[bytes],
        quality: int = 80,
        lossless: bool = False,
        apply_optimizers: bool = True,
    ) -> AvifImageFile: ...
    def save_as_ico(self, f: IO[bytes], apply_optimizers: bool = True) -> IcoImageFile: ...
    def auto_orient(self) -> PillowImage: ...
    def get_pillow_image(self) -> _PILImage: ...
    @classmethod
    def open(cls, image_file: IO[bytes] | ImageFile) -> PillowImage: ...
    def to_buffer_rgb(self) -> RGBImageBuffer: ...
    def to_buffer_rgba(self) -> RGBAImageBuffer: ...

willow_image_classes: list[type[PillowImage]]



from collections.abc import Callable
from typing import IO, NamedTuple, Self
from xml.etree.ElementTree import Element, ElementTree
import re

from .image import Image, SvgImageFile

class WillowSvgException(Exception): ...
class InvalidSvgAttribute(WillowSvgException): ...
class InvalidSvgSizeAttribute(WillowSvgException): ...
class SvgViewBoxParseError(WillowSvgException): ...

class ViewBox(NamedTuple):
    min_x: float
    min_y: float
    width: float
    height: float

def view_box_to_attr_str(view_box: ViewBox) -> str: ...

class ViewportToUserSpaceTransform:
    scale_x: float
    scale_y: float
    translate_x: float
    translate_y: float
    def __init__(
        self,
        scale_x: float,
        scale_y: float,
        translate_x: float,
        translate_y: float,
    ) -> None: ...
    def __eq__(self, other: object) -> bool: ...
    def __call__(self, rect: tuple[float, float, float, float]) -> tuple[float, float, float, float]: ...

def get_viewport_to_user_space_transform(
    svg: SvgImage,
) -> ViewportToUserSpaceTransform: ...

class SvgWrapper:
    UNIT_RE: re.Pattern[str]
    NUMBER_PATTERN: str
    VIEW_BOX_RE: re.Pattern[str]
    PRESERVE_ASPECT_RATIO_RE: re.Pattern[str]
    COEFFICIENTS: dict[str, float]
    dom: ElementTree
    dpi: int
    font_size_px: int
    view_box: ViewBox
    preserve_aspect_ratio: str
    width: float
    height: float
    def __init__(self, dom: ElementTree, dpi: int = 96, font_size_px: int = 16) -> None: ...
    def __copy__(self) -> Self: ...
    @classmethod
    def from_file(cls, f: IO[bytes]) -> Self: ...
    @property
    def root(self) -> Element: ...
    def set_root_attr(self, attr: str, value: str | float) -> None: ...
    def set_width(self, width: float) -> None: ...
    def set_height(self, height: float) -> None: ...
    def set_view_box(self, view_box: ViewBox) -> None: ...
    def write(self, f: IO[bytes]) -> None: ...

class SvgImage(Image):
    image: SvgWrapper
    def __init__(self, image: SvgWrapper) -> None: ...
    def crop(
        self,
        rect: tuple[float, float, float, float],
        get_transformer: Callable[[SvgImage], ViewportToUserSpaceTransform] = ...,
    ) -> SvgImage: ...
    def resize(self, size: tuple[int, int]) -> SvgImage: ...
    def get_size(self) -> tuple[float, float]: ...
    def auto_orient(self) -> Self: ...
    def has_animation(self) -> bool: ...
    def get_frame_count(self) -> int: ...
    def write(self, f: IO[bytes]) -> None: ...
    def save_as_svg(self, f: IO[bytes]) -> SvgImageFile: ...
    @classmethod
    def open(cls, svg_image_file: IO[bytes] | SvgImageFile) -> SvgImage: ...



from collections.abc import Callable
from io import BytesIO
from tempfile import SpooledTemporaryFile
from typing import IO, Any, TypeVar
from xml.etree.ElementTree import ElementTree

from .registry import registry as registry

_F = TypeVar("_F", bound=Callable[..., Any])

class UnrecognisedImageFormatError(IOError): ...
class BadImageOperationError(ValueError): ...

class Image:
    @classmethod
    def check(cls) -> None: ...
    @staticmethod
    def operation(func: _F) -> _F: ...
    @staticmethod
    def converter_to(to_class: type[Image], cost: int | None = None) -> Callable[[_F], _F]: ...
    @staticmethod
    def converter_from(from_class: type[Image] | list[type[Image]], cost: int | None = None) -> Callable[[_F], _F]: ...
    def __getattr__(self, attr: str) -> Callable[..., Any]: ...
    @classmethod
    def open(cls, f: IO[bytes]) -> Image: ...
    @classmethod
    def maybe_xml(cls, f: IO[bytes]) -> bool: ...
    def save(
        self,
        image_format: str,
        output: IO[bytes],
        apply_optimizers: bool = True,
    ) -> ImageFile | None: ...
    def optimize(
        self,
        image_file: SpooledTemporaryFile[bytes] | BytesIO | str | bytes | IO[bytes],
        image_format: str,
    ) -> None: ...

class ImageBuffer(Image):
    size: tuple[int, int]
    data: bytes
    def __init__(self, size: tuple[int, int], data: bytes) -> None: ...
    def get_size(self) -> tuple[int, int]: ...

class RGBImageBuffer(ImageBuffer):
    mode: str
    def has_alpha(self) -> bool: ...
    def has_animation(self) -> bool: ...

class RGBAImageBuffer(ImageBuffer):
    mode: str
    def has_alpha(self) -> bool: ...
    def has_animation(self) -> bool: ...

class ImageFile(Image):
    @property
    def format_name(self) -> str: ...
    @property
    def mime_type(self) -> str: ...
    f: IO[bytes]
    def __init__(self, f: IO[bytes]) -> None: ...

class JPEGImageFile(ImageFile):
    @property
    def format_name(self) -> str: ...
    @property
    def mime_type(self) -> str: ...

class PNGImageFile(ImageFile):
    @property
    def format_name(self) -> str: ...
    @property
    def mime_type(self) -> str: ...

class GIFImageFile(ImageFile):
    @property
    def format_name(self) -> str: ...
    @property
    def mime_type(self) -> str: ...

class BMPImageFile(ImageFile):
    @property
    def format_name(self) -> str: ...
    @property
    def mime_type(self) -> str: ...

class TIFFImageFile(ImageFile):
    @property
    def format_name(self) -> str: ...
    @property
    def mime_type(self) -> str: ...

class WebPImageFile(ImageFile):
    @property
    def format_name(self) -> str: ...
    @property
    def mime_type(self) -> str: ...

class SvgImageFile(ImageFile):
    format_name: str
    mime_type: str
    dom: ElementTree
    def __init__(self, f: IO[bytes], dom: ElementTree | None = None) -> None: ...

class HeicImageFile(ImageFile):
    @property
    def format_name(self) -> str: ...
    @property
    def mime_type(self) -> str: ...

class AvifImageFile(ImageFile):
    @property
    def format_name(self) -> str: ...
    @property
    def mime_type(self) -> str: ...

class IcoImageFile(ImageFile):
    format_name: str
    mime_type: str

INITIAL_IMAGE_CLASSES: dict[str, type[ImageFile]]



from typing import ClassVar
import logging

logger: logging.Logger

class OptimizerBase:
    library_name: ClassVar[str]
    image_format: ClassVar[str]
    class Meta:
        abstract: bool

    @classmethod
    def applies_to(cls, image_format: str) -> bool: ...
    @classmethod
    def get_check_library_arguments(cls) -> list[str]: ...
    @classmethod
    def check_library(cls) -> bool: ...
    @classmethod
    def get_command_arguments(cls, file_path: str) -> list[str]: ...
    @classmethod
    def process(cls, file_path: str) -> None: ...



from numpy import ndarray
from willow.image import Image, RGBImageBuffer

class BaseOpenCVImage(Image):
    image: ndarray
    size: tuple[int, int]
    def __init__(self, image: ndarray, size: tuple[int, int]) -> None: ...
    @classmethod
    def check(cls) -> None: ...
    def get_size(self) -> tuple[int, int]: ...
    def get_frame_count(self) -> int: ...
    def has_alpha(self) -> bool: ...
    def has_animation(self) -> bool: ...

class OpenCVColorImage(BaseOpenCVImage):
    @classmethod
    def check(cls) -> None: ...
    @classmethod
    def from_buffer_rgb(cls, image_buffer: RGBImageBuffer) -> OpenCVColorImage: ...

class OpenCVGrayscaleImage(BaseOpenCVImage):
    face_haar_flags: int
    face_min_neighbors: int
    face_haar_scale: float
    face_min_size: tuple[int, int]
    def detect_features(self) -> list[list[float]]: ...
    def detect_faces(
        self, cascade_filename: str = "haarcascade_frontalface_alt2.xml"
    ) -> list[tuple[int, int, int, int]]: ...
    @classmethod
    def from_color(cls, colour_image: OpenCVColorImage) -> OpenCVGrayscaleImage: ...

willow_image_classes: list[type[BaseOpenCVImage]]



from collections.abc import Callable, Generator
from typing import Any
import types

from .image import Image
from .optimizers.base import OptimizerBase

type _ConverterFunc = Callable[..., Image]
type _OperationFunc = Callable[..., Any]
type _PathStep = tuple[_ConverterFunc, type[Image]]

class UnrecognisedOperationError(LookupError): ...
class UnavailableOperationError(LookupError): ...
class UnroutableOperationError(LookupError): ...

class WillowRegistry:
    def __init__(self) -> None: ...
    def register_operation(
        self,
        image_class: type[Image],
        operation_name: str,
        func: _OperationFunc,
    ) -> None: ...
    def register_converter(
        self,
        from_image_class: type[Image],
        to_image_class: type[Image],
        func: _ConverterFunc,
        cost: int | None = None,
    ) -> None: ...
    def register_image_class(self, image_class: type[Image]) -> None: ...
    def register_plugin(self, plugin: types.ModuleType) -> None: ...
    def register_optimizer(self, optimizer_class: type[OptimizerBase]) -> None: ...
    def get_operation(self, image_class: type[Image], operation_name: str) -> _OperationFunc: ...
    def operation_exists(self, operation_name: str) -> bool: ...
    def get_converter(self, from_image_class: type[Image], to_image_class: type[Image]) -> _ConverterFunc: ...
    def get_converter_cost(self, from_image_class: type[Image], to_image_class: type[Image]) -> int: ...
    def get_image_classes(
        self,
        with_operation: str | None = None,
        available: bool | None = None,
    ) -> set[type[Image]]: ...
    def get_optimizers_for_format(self, image_format: str) -> list[type[OptimizerBase]]: ...
    def get_converters_from(self, from_image_class: type[Image]) -> Generator[_PathStep]: ...
    def find_all_paths(
        self,
        start: type[Image],
        end: type[Image],
        path: list[_PathStep] = ...,
        seen_classes: set[type[Image]] = ...,
    ) -> list[list[_PathStep]]: ...
    def get_path_cost(
        self,
        start: type[Image],
        path: list[_PathStep],
    ) -> int: ...
    def find_shortest_path(self, start: type[Image], end: type[Image]) -> tuple[list[_PathStep] | None, int | None]: ...
    def find_closest_image_class(
        self, start: type[Image], image_classes: set[type[Image]]
    ) -> tuple[
        type[Image] | None,
        list[_PathStep] | None,
        int | None,
    ]: ...
    def find_operation(
        self, from_class: type[Image], operation_name: str
    ) -> tuple[
        _OperationFunc,
        type[Image],
        list[_PathStep],
        int,
    ]: ...

registry: WillowRegistry



from .cwebp import Cwebp as Cwebp
from .gifsicle import Gifsicle as Gifsicle
from .jpegoptim import Jpegoptim as Jpegoptim
from .optipng import Optipng as Optipng
from .pngquant import Pngquant as Pngquant



from typing import ClassVar

from .base import OptimizerBase

__all__ = ["Pngquant"]

class Pngquant(OptimizerBase):
    library_name: ClassVar[str]
    image_format: ClassVar[str]
    @classmethod
    def get_command_arguments(cls, file_path: str, progressive: bool = False) -> list[str]: ...  # type: ignore[override]



from typing import ClassVar

from .base import OptimizerBase

__all__ = ["Gifsicle"]

class Gifsicle(OptimizerBase):
    library_name: ClassVar[str]
    image_format: ClassVar[str]
    @classmethod
    def get_command_arguments(cls, file_path: str) -> list[str]: ...
